/**********************************************************************
Copyright (c) 2008 Erik Bengtson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.datanucleus.store.json;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URLConnection;
import java.util.Properties;

import org.datanucleus.ObjectManager;
import org.datanucleus.StateManager;
import org.datanucleus.exceptions.NucleusDataStoreException;
import org.datanucleus.exceptions.NucleusException;
import org.datanucleus.exceptions.NucleusObjectNotFoundException;
import org.datanucleus.store.StoreManager;
import org.datanucleus.store.StorePersistenceHandler;
import org.datanucleus.store.connection.ManagedConnection;
import org.datanucleus.store.json.fieldmanager.FetchFieldManager;
import org.datanucleus.store.json.fieldmanager.InsertFieldManager;
import org.json.JSONException;
import org.json.JSONObject;

public class JsonPersistenceHandler implements StorePersistenceHandler
{
    private JsonStoreManager storeMgr;

    JsonPersistenceHandler(StoreManager storeMgr)
    {
        this.storeMgr = (JsonStoreManager)storeMgr;
    }
    
    public void close()
    {
        // TODO Auto-generated method stub
    }

    public void insertObject(StateManager sm)
    {
        // Check if read-only so update not permitted
        storeMgr.assertReadOnlyForUpdateOfObject(sm);

        int[] fieldNumbers = sm.getClassMetaData().getAllMemberPositions();
        String url = sm.getClassMetaData().getValueForExtension("url");
        Properties options = new Properties();
        if (url!=null)
        {
            options.put(ConnectionFactoryImpl.STORE_JSON_URL, url);
        }
        else
        {
            options.put(ConnectionFactoryImpl.STORE_JSON_URL, sm.getClassMetaData().getFullClassName());
        }        
        ManagedConnection mconn = storeMgr.getConnection(sm.getObjectManager(), options);
        URLConnection conn = (URLConnection) mconn.getConnection();
        JSONObject jsonobj = new JSONObject();

        sm.provideFields(fieldNumbers, new InsertFieldManager(sm, jsonobj));
        sm.provideFields(sm.getClassMetaData().getPKMemberPositions(), new InsertFieldManager(sm, jsonobj));

        write("POST",conn.getURL().toExternalForm(),conn,jsonobj);
    }

    public void updateObject(StateManager sm, int[] fieldNumbers)
    {
        // Check if read-only so update not permitted
        storeMgr.assertReadOnlyForUpdateOfObject(sm);

        String url = sm.getClassMetaData().getValueForExtension("url");
        Properties options = new Properties();
        if (url!=null)
        {
            options.put(ConnectionFactoryImpl.STORE_JSON_URL, url);
        }
        else
        {
            options.put(ConnectionFactoryImpl.STORE_JSON_URL, sm.getClassMetaData().getFullClassName());
        }        
        ManagedConnection mconn = storeMgr.getConnection(sm.getObjectManager(), options);
        URLConnection conn = (URLConnection) mconn.getConnection();
        JSONObject jsonobj = new JSONObject();

        sm.provideFields(fieldNumbers, new InsertFieldManager(sm, jsonobj));
        sm.provideFields(sm.getClassMetaData().getPKMemberPositions(), new InsertFieldManager(sm, jsonobj));

        write("PUT",conn.getURL().toExternalForm(),conn,jsonobj);
    }

    public void deleteObject(StateManager sm)
    {
        // Check if read-only so update not permitted
        storeMgr.assertReadOnlyForUpdateOfObject(sm);

        JSONObject jsonobj = new JSONObject();
        sm.provideFields(sm.getClassMetaData().getPKMemberPositions(), new InsertFieldManager(sm,jsonobj));
        String url = sm.getClassMetaData().getValueForExtension("url");
        try
        {
            if (url==null)
            {
                url = "";
            }
            url += jsonobj.get(JSONObject.getNames(jsonobj)[0]).toString();
        }
        catch (JSONException e)
        {
            throw new NucleusException(e.getMessage(), e);
        }
        Properties options = new Properties();
        if (url!=null)
        {
            options.put(ConnectionFactoryImpl.STORE_JSON_URL, url);
        }
        else
        {
            options.put(ConnectionFactoryImpl.STORE_JSON_URL, sm.getClassMetaData().getFullClassName());
        }        
        ManagedConnection mconn = storeMgr.getConnection(sm.getObjectManager(), options);
        URLConnection conn = (URLConnection) mconn.getConnection();
        try
        {
            int length = jsonobj.toString().length();
            HttpURLConnection http = (HttpURLConnection)conn;
            http.setRequestProperty("Content-Length", ""+length);
            http.setRequestMethod("DELETE");
            http.setReadTimeout(10000);
            http.connect();
            http.getResponseCode();
        }
        catch (IOException e)
        {
            throw new NucleusDataStoreException(e.getMessage(),e);
        }
    }

    public void fetchObject(StateManager sm, int[] fieldNumbers)
    {
        JSONObject jsonobj = new JSONObject();
        sm.provideFields(sm.getClassMetaData().getPKMemberPositions(), new InsertFieldManager(sm,jsonobj));
        String url = sm.getClassMetaData().getValueForExtension("url");
        try
        {
            if (url==null)
            {
                url = "";
            }
            url += jsonobj.get(JSONObject.getNames(jsonobj)[0]).toString();
        }
        catch (JSONException e)
        {
            throw new NucleusException(e.getMessage(), e);
        }
        Properties options = new Properties();
        if (url!=null)
        {
            options.put(ConnectionFactoryImpl.STORE_JSON_URL, url);
        }
        else
        {
            options.put(ConnectionFactoryImpl.STORE_JSON_URL, sm.getClassMetaData().getFullClassName());
        }        
        ManagedConnection mconn = storeMgr.getConnection(sm.getObjectManager(), options);
        URLConnection conn = (URLConnection) mconn.getConnection();
        JSONObject result = read("GET",conn.getURL().toExternalForm(),conn,jsonobj);
        
        sm.replaceFields(fieldNumbers, new FetchFieldManager(sm,result));

    }

    public Object findObject(ObjectManager om, Object id)
    {
        // TODO Auto-generated method stub
        return null;
    }

    public void locateObject(StateManager sm)
    {
        JSONObject jsonobj = new JSONObject();
        sm.provideFields(sm.getClassMetaData().getPKMemberPositions(), new InsertFieldManager(sm,jsonobj));
        String url = sm.getClassMetaData().getValueForExtension("url");
        try
        {
            if (url==null)
            {
                url = "";
            }
            url += jsonobj.get(JSONObject.getNames(jsonobj)[0]).toString();
        }
        catch (JSONException e)
        {
            throw new NucleusException(e.getMessage(), e);
        }
        Properties options = new Properties();
        if (url!=null)
        {
            options.put(ConnectionFactoryImpl.STORE_JSON_URL, url);
        }
        else
        {
            options.put(ConnectionFactoryImpl.STORE_JSON_URL, sm.getClassMetaData().getFullClassName());
        }        
        ManagedConnection mconn = storeMgr.getConnection(sm.getObjectManager(), options);
        URLConnection conn = (URLConnection) mconn.getConnection();

        try
        {
            HttpURLConnection http = (HttpURLConnection)conn;
            http.setDoOutput(true);
            http.setRequestMethod("HEAD");
            http.setReadTimeout(10000);
            http.connect();
            int code = http.getResponseCode();
            if (code == 404)
            {
                throw new NucleusObjectNotFoundException();
            }
        }
        catch (IOException e)
        {
            throw new NucleusObjectNotFoundException(e.getMessage(),e);
        }
    }

    private void write(String method, String requestUri, URLConnection conn, JSONObject jsonobj)
    {
        try
        {
            if (JSONLogger.LOGGER.isDebugEnabled())
            {
                JSONLogger.LOGGER.debug("Writing to URL "+requestUri+" content "+jsonobj.toString());
            }
            int length = jsonobj.toString().length();
            HttpURLConnection http = (HttpURLConnection)conn;
            http.setRequestProperty("Content-Length", ""+length);
            http.setDoOutput(true);
            http.setRequestMethod(method);
            http.setReadTimeout(10000);
            http.connect();
            OutputStream os = conn.getOutputStream();
            os.write(jsonobj.toString().getBytes());
            os.flush();
            os.close();
            http.getResponseCode();
        }
        catch (IOException e)
        {
            throw new NucleusDataStoreException(e.getMessage(),e);
        }
    }

    private JSONObject read(String method, String requestUri, URLConnection conn, JSONObject jsonobj)
    {
        try
        {
            int length = jsonobj.toString().length();
            HttpURLConnection http = (HttpURLConnection)conn;
            http.setRequestProperty("Content-Length", ""+length);
            //http.setDoOutput(true);
            http.setDoInput(true);
            http.setRequestMethod(method);
            http.setReadTimeout(10000);
            http.connect();
            /*
            OutputStream os = conn.getOutputStream();
            os.write(json.toString().getBytes());
            os.flush();
            os.close();
            */
            int code = http.getResponseCode();
            if (code == 404)
            {
                throw new NucleusObjectNotFoundException();
            }
            /*String msg =*/ http.getResponseMessage();
            StringBuffer sb = new StringBuffer();
            if (http.getContentLength()>0)
            {
                for( int i=0; i<http.getContentLength(); i++)
                {
                    sb.append((char)http.getInputStream().read());
                }
            }
            else
            {
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                byte[] buffer = new byte[1024];
                int r;
                while ((r = http.getInputStream().read(buffer)) != -1) {
                    baos.write(buffer, 0, r);
                }
                
                sb.append(new String(baos.toByteArray()));        
            }
            http.getInputStream().close();
            return new JSONObject(sb.toString());
        }
        catch (IOException e)
        {
            throw new NucleusDataStoreException(e.getMessage(),e);
        }
        catch (JSONException e)
        {
            throw new NucleusDataStoreException(e.getMessage(),e);
        }
    }
}